## -*- coding: utf-8; -*-
<%inherit file="/master/view.mako" />

<%def name="extra_javascript()">
  ${parent.extra_javascript()}
  % if not use_buefy:
  <script type="text/javascript">

    function show_packages(type) {
        if (type == 'all') {
            $('.showing .diffs').css('font-weight', 'normal');
            $('table.diff tbody tr').show();
            $('.showing .all').css('font-weight', 'bold');
        } else if (type == 'diffs') {
            $('.showing .all').css('font-weight', 'normal');
            $('table.diff tbody tr:not(.diff)').hide();
            $('.showing .diffs').css('font-weight', 'bold');
        }
    }

    $(function() {

        show_packages('diffs');

        $('.showing .all').click(function() {
            show_packages('all');
            return false;
        });

        $('.showing .diffs').click(function() {
            show_packages('diffs')
            return false;
        });

    });

  </script>
  % endif
</%def>

<%def name="extra_styles()">
  ${parent.extra_styles()}
  <style type="text/css">
    .progress-with-textout {
        border: 1px solid Black;
        line-height: 1.2;
        overflow: auto;
        padding: 1rem;
    }
  </style>
</%def>

<%def name="render_this_page()">
  ${parent.render_this_page()}

  % if master.has_perm('execute'):
      ${h.form(master.get_action_url('declare_failure', instance), ref='declareFailureForm')}
      ${h.csrf_token(request)}
      ${h.end_form()}
  % endif
</%def>

<%def name="render_buefy_form()">
  <div class="form">
    <${form.component}
      % if master.has_perm('execute'):
      @declare-failure="declareFailure"
      % endif
      >
    </${form.component}>
  </div>
</%def>

<%def name="render_form_buttons()">
  % if not instance.executed and instance.status_code == enum.UPGRADE_STATUS_PENDING and master.has_perm('execute'):
      <div class="buttons">
        % if instance.enabled and not instance.executing:
            % if use_buefy and expose_websockets:
                <b-button type="is-primary"
                          icon-pack="fas"
                          icon-left="arrow-circle-right"
                          :disabled="upgradeExecuting"
                          @click="executeUpgrade()">
                  {{ upgradeExecuting ? "Working, please wait..." : "Execute this upgrade" }}
                </b-button>
            % elif use_buefy:
                ${h.form(url('{}.execute'.format(route_prefix), uuid=instance.uuid), **{'@submit': 'submitForm'})}
                ${h.csrf_token(request)}
                <b-button type="is-primary"
                          native-type="submit"
                          icon-pack="fas"
                          icon-left="arrow-circle-right"
                          :disabled="formSubmitting">
                  {{ formSubmitting ? "Working, please wait..." : "Execute this upgrade" }}
                </b-button>
                ${h.end_form()}
            % else:
                ${h.form(url('{}.execute'.format(route_prefix), uuid=instance.uuid), class_='autodisable')}
                ${h.csrf_token(request)}
                ${h.submit('execute', "Execute this upgrade", class_='button is-primary')}
                ${h.end_form()}
            % endif
        % elif instance.enabled:
            <button type="button" class="button is-primary" disabled="disabled" title="This upgrade is currently executing">Execute this upgrade</button>
        % else:
            <button type="button" class="button is-primary" disabled="disabled" title="This upgrade is not enabled">Execute this upgrade</button>
        % endif
      </div>

      <b-modal :active.sync="upgradeExecuting"
               full-screen
               :can-cancel="false">
        <div class="card">
          <div class="card-content">

            <div class="level">
              <div class="level-item has-text-centered"
                   style="display: flex; flex-direction: column;">
                <p class="block">Upgrading (please wait) ...</p>
                <b-progress size="is-large"
                            style="width: 400px;"
##                             :value="80"
##                             show-value
##                             format="percent"
                            >
                </b-progress>
              </div>
              <div class="level-right">
                <div class="level-item">
                  <b-button type="is-warning"
                            icon-pack="fas"
                            icon-left="sad-tear"
                            @click="declareFailureClick()">
                    Declare Failure
                  </b-button>
                </div>
              </div>
            </div>

            <div class="container progress-with-textout is-family-monospace is-size-7"
                 ref="textout">
              <span v-for="line in progressOutput"
                    :key="line.key"
                    v-html="line.text">
              </span>

              ## nb. we auto-scroll down to "see" this element
              <div ref="seeme"></div>
            </div>

          </div>
        </div>
      </b-modal>

  % endif
</%def>

<%def name="modify_this_page_vars()">
  ${parent.modify_this_page_vars()}
  <script type="text/javascript">

    TailboneFormData.showingPackages = 'diffs'

    % if master.has_perm('execute'):

        % if expose_websockets:

            TailboneFormData.upgradeExecuting = ${json.dumps(instance.executing)|n}
            TailboneFormData.progressOutput = []
            TailboneFormData.progressOutputCounter = 0

            TailboneForm.methods.executeUpgrade = function() {
                this.upgradeExecuting = true

                // grow the textout area to fill most of screen
                this.$nextTick(() => {
                    let textout = this.$refs.textout
                    let height = window.innerHeight - textout.offsetTop - 50
                    textout.style.height = height + 'px'
                })

                let url = '${master.get_action_url('execute', instance)}'
                this.submitForm(url, {ws: true}, response => {

                    ## TODO: should be a cleaner way to get this url?
                    url = '${request.route_url('ws.upgrades.execution_progress', _query={'uuid': instance.uuid})}'
                    url = url.replace(/^https?:/, 'wss:')

                    this.ws = new WebSocket(url)
                    let that = this

                    ## TODO: add support for this here?
                    // this.ws.onclose = (event) => {
                    //     // websocket closing means 1 of 2 things:
                    //     // - user navigated away from page intentionally
                    //     // - server connection was broken somehow
                    //     // only one of those is "bad" and we only want to
                    //     // display warning in 2nd case.  so we simply use a
                    //     // brief delay to "rule out" the 1st scenario
                    //     setTimeout(() => { that.websocketBroken = true },
                    //                3000)
                    // }

                    this.ws.onmessage = (event) => {
                        let data = JSON.parse(event.data)

                        if (data.complete) {

                            // upgrade has completed; reload page to view result
                            location.reload()

                        } else if (data.stdout) {

                            // add lines to textout area
                            that.progressOutput.push({
                                key: ++that.progressOutputCounter,
                                text: data.stdout})

                            //  scroll down to end of textout area
                            this.$nextTick(() => {
                                this.$refs.seeme.scrollIntoView()
                            })
                        }
                    }
                })
            }

        % else:
            ## no websockets

            TailboneFormData.formSubmitting = false

            TailboneForm.methods.submitForm = function() {
                this.formSubmitting = true
            }

        % endif

        TailboneFormData.declareFailureSubmitting = false

        TailboneForm.methods.declareFailureClick = function() {
            if (confirm("Really declare this upgrade a failure?")) {
                this.declareFailureSubmitting = true
                this.$emit('declare-failure')
            }
        }

        ThisPage.methods.declareFailure = function() {
            this.$refs.declareFailureForm.submit()
        }

    % endif

  </script>
</%def>


${parent.body()}
